(*
$Id: teui.pas 1.7 2002/03/19 08:26:33 tony Exp tony $
$Log: teui.pas $
Revision 1.7  2002/03/19 08:26:33  tony
Use main executable to read resource information

Revision 1.6  2002/01/15 14:58:26  tony
Fixed code to allow opening of blank file if no filename
listed. Thanks to Ralf Quint for pointing this flaw out to me

Revision 1.5  2001/08/28 16:41:16  tony
Added hint status line and ability to read resource file for strings

Revision 1.4  2001/08/26 02:53:44  tony
updated status line code

Revision 1.3  2001/08/24 07:23:38  tony
Fixed command constants, cleaned up source

Revision 1.2  2001/08/23 07:03:48  tony
Added procedure for Color selection dialog and added menu item for
repainting desktop

Revision 1.1  2001/08/16 21:36:38  tony
Cleaned up source code

Revision 1.0  2001/08/15 18:38:00  tony
Initial revision


        Tony's Full Screen Editor
        Version 1.00

        Copyright (C) AGE Software
        All rights reserved

        Purpose: Full screen multi-purpose editor.
        FILENAME: TEUI.PAS

        Contains procedures and functions related to the
        user interface portion of TE such as informational
        messages, dialog boxes, window and screen management, etc.

*)
unit TEUI;

interface

uses App, Editors, StdDlg, Dialogs, Drivers;

type tEditorApp = object(TApplication)
        AboutBox : PDialog;
        constructor Init;
        procedure InitStatusLine; virtual;
        procedure InitMenuBar; virtual;
        procedure HandleEvent(var Event : TEvent); virtual;
        procedure DoAboutBox;
        procedure ColorSelDialog;
        procedure OutofMemory; virtual;
     end;

implementation

uses Strings, ColorSel, TeDos, Memory, MsgBox, Dos, Objects, Menus, Views, TEConst;

const
  WinNum : byte = 1;    { Default window count }
  HeapSize = 64 * (1024 div 16);  { 4096 byte heapsize }
  UIRCSID : String = '$Id: teui.pas 1.7 2002/03/19 08:26:33 tony Exp tony $';
  UIRCSRev : String = '$Revision: 1.7 $';

type PHintStatusLine  = ^THintStatusLine;
     THintStatusLine  = object(TStatusLine)
        function Hint(AHelpCtx : Word) : String; virtual;
     end;

type PProtectedStream = ^TProtectedStream;
     TProtectedStream = object(TBufStream)
        procedure Error(Code, Info : Integer); virtual;
     end;

var ClipWindow : PEditWindow;  (* Create clipboard window *)
    RezFile : TResourceFile;
    RezStream : PStream;
    StrList : PStringList;

(*

Error heandler for Stream Errors. This will prevent program lockups when
the resource file can't be found

procedure EditStreamError var (S : TStream); far;
begin
end.
*)

function OpenEditor(theFile: FNameStr; Visible: Boolean): PEditWindow;
{
The OpenEditor is used by the event handler to actually open files. It can
generate a new editor window, or create an editor window for an existing file
}

var P: PWindow;
    R: TRect;
begin
  DeskTop^.GetExtent(R);
  P := New(PEditWindow, Init(R, theFile, WinNum));
  if not Visible then P^.Hide;
  OpenEditor := PEditWindow(Application^.InsertWindow(P));
  inc(WinNum);
end;

procedure FileOpen(theFile : FNameStr);
begin
  if Application^.ExecuteDialog(New(PFileDialog, Init(theFile, 'Open file',
  '~N~ame', fdOpenButton or fdHelpButton, 100)), @theFile) <> cmCancel then
  OpenEditor(theFile, True);
end;

function THintStatusLine.Hint(AHelpCtx : Word) : String;
begin
  case AHelpCtx of
    hcHintAbout  : Hint := StrList^.Get(sHintAbout);
    hcAccessMenu : Hint := StrList^.Get(sAccessMenu);
    hcRepaint    : Hint := StrList^.Get(sRepaint);
    hcFileMenu   : Hint := StrList^.Get(sFileMenu);
    hcEditMenu   : Hint := StrList^.Get(sEditMenu);
    hcClipShow   : Hint := StrList^.Get(sClipShow);
  else
    Hint := '';
  end;
end;
procedure TProtectedStream.Error(Code, Info : Integer);
var
  c, i : string;
begin
  str(code, c);
  str(info, i);
  DoneSysError;
  DoneEvents;
  DoneVideo;
  DoneMemory;
  PrintStr('Error in stream: Code '+c+' Info '+i+#13#10);
  Halt(1);
end;
{
  Turbo Vision Code begins here for the user interface
}
constructor TEditorApp.Init;
{
        Application initialization, sets up the heapsize and the clipboard
}

  procedure processcmdline;
  var
    I : Byte;
    P : String;

  begin
    for i := 1 to paramcount do
    begin
      P := paramstr(i);
      if (P[1] = '/') or (P[1] = '-') then
      else begin
         if P[Length(P)] = '\' then
           P := P + '*.*';
         if (Pos('?', P) = 0) and (Pos('*', P) = 0) then
           OpenEditor(FExpand(P), True)
         else
           FileOpen(P);
      end;
    end;
  end;

begin
  MaxHeapSize := HeapSize;
  RezStream := New(PProtectedStream, Init('TE.EXE', stOpenRead, 4096));
  RezFile.Init(RezStream);
  RegisterType(RStringList);
  StrList := PStringList(RezFile.Get('Strings'));
  ParseCmdLine;
  inherited Init;
  EditorDialog := StdEditorDialog;
  DisableCommands([cmSave, cmSaveAs, cmCut, cmCopy, cmPaste, cmClear,
    cmUndo, cmFind, cmReplace, cmSearchAgain, cmCloseAll]);
  EditorDialog := StdEditorDialog;
  ClipWindow := OpenEditor('', False);
  dec(WinNum);
  if ClipWindow <> nil then
  begin
    Clipboard := ClipWindow^.Editor;
    Clipboard^.CanUndo := False;
  end;
  if paramcount > 0 then
    ProcessCmdLine
  else
   OpenEditor('', true);
  Cascade; { automatically cascade windows if more than one is opened }
end;

procedure TEditorApp.ColorSelDialog;
var
  R : TRect;
  D : PDialog;

begin
  D := New(PColorDialog, Init('',
    ColorGroup('Desktop',          DesktopColorItems(nil),
    ColorGroup('Menus',            MenuColorItems(nil),
    ColorGroup('Editor/Windows',   WindowColorItems(wpBlueWindow, nil),
    ColorGroup('Dialog Boxes',     DialogColorItems(dpGrayDialog, nil),
    nil))))));
  if ExecuteDialog(D, Application^.GetPalette) <> cmCancel then
    begin
      DoneMemory;
      ReDraw;
    end;

end;

procedure TEditorApp.InitStatusLine;
var R : TRect;
begin
  GetExtent(R);                         { get application boundaries }
  R.A.Y := R.B.Y - 1;                   { set top to one above bottom }
  StatusLine := New(PHintStatusLine, Init(R,
    NewStatusDef(0, 999,
      NewStatusKey('~F1~ Help', kbF1, cmHelp,
      NewStatusKey('~F2~ Save', kbF2, cmSave,
      NewStatusKey('~F3~ Open', kbF3, cmOpen,
      NewStatusKey('~Alt+F3~ Close', kbAltF3, cmClose,
      StdStatusKeys(nil))))),
    NewStatusDef(1000, $FFFF,
      NewStatusKey('~F1~ Help', kbF1, cmHelp,
      StdStatusKeys(nil)), nil))));
end;

procedure TEditorApp.InitMenuBar;
var R : TRect;
begin
  GetExtent(R);
  R.B.Y := R.A.Y + 1;
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewSubMenu('~'#240'~',hcAccessMenu, NewMenu(
      NewItem('~R~epaint Desktop', '', kbNoKey, cmRepaint, hcRepaint,
      nil)),
    NewSubMenu('~F~ile', hcFileMenu, NewMenu(
      StdFileMenuItems(nil)),
    NewSubMenu('~E~dit', hcEditMenu, NewMenu(
      StdEditMenuItems(
      NewLine(
      NewItem('~S~how clipboard', '', kbNoKey, cmClipShow, hcClipShow,
        nil)))),
    NewSubMenu('~S~earch', hcSearchMenu, NewMenu(
      NewItem('~F~ind...', '', kbNoKey, cmFind, hcFind,
      NewItem('~R~eplace...', '', kbNoKey, cmReplace, hcReplace,
      NewItem('~S~earch again', '', kbNoKey, cmSearchAgain, hcSearchAgain,
      nil)))),
    NewSubMenu('~O~ptions', hcOptionsMenu, NewMenu(
      NewItem('~T~oggle video', '', kbNoKey, cmOptionsVideo, hcOptionsVideo,
      NewItem('~C~olors...', '', kbNoKey, cmColors, hcColors,
    nil))),
    NewSubMenu('~W~indow', hcWindowMenu, NewMenu(
      StdWindowMenuItems(nil)),
    NewSubMenu('~H~elp', hcHelpMenu, NewMenu(
      NewItem('~A~bout...', '', kbNoKey, cmAbout, hcAbout, nil)),
    nil))))))))));
end;

procedure TEditorApp.DoAboutBox;

var
  S : String;
  L : Longint;
  R : TRect;
begin
  R.Assign(0, 0, 40, 11);
  AboutBox := New(PDialog, Init(R, 'About TE'));
  with AboutBox^ do begin
    HelpCtx := hcHintAbout;
    Options := Options or ofCentered;
    R.Assign(4, 2, 36, 4);
    FormatStr(S, #3'TE - Version %d.%d ', VERSION);
    S := S + Copy(RCSRev, 2, length(RCSRev) - 2);
    Insert(New(PStaticText, Init(R, S)));
    R.Assign(4, 4, 36, 6);
    Insert(New(PStaticText, Init(R, #3'Copyright (C) AGE Software')));
    R.Assign(4, 6, 36, 8);
    L := MemAvail;
    FormatStr(S, #3'Memory Available: %-5d bytes', L);
    Insert(New(PStaticText, Init(R, S)));
    R.Assign(15, 8, 25, 10);
    Insert(New(PButton, Init(R, '~C~lose', cmOK, bfDefault)));
  end;
  ExecuteDialog(AboutBox, nil);

end;

procedure TEditorApp.HandleEvent;
(*
  Event handler code
*)

  procedure RepaintDesktop;
  begin
    DoneMemory;
    ClearScreen;
    Redraw;
  end;

  procedure ShowClip;
  begin
    ClipWindow^.Select;
    ClipWindow^.Show;
  end;


begin
  inherited HandleEvent(Event);
  if Event.What = evCommand then
  case Event.Command of
    cmOptionsVideo: SetScreenMode(ScreenMode xor smFont8x8);
    cmColors : ColorSelDialog;
    cmNew:OpenEditor('',true);
    cmRepaint : RepaintDesktop;
    cmClipShow : ShowClip;
    cmChangeDir : ExecuteDialog(New(PChDirDialog, Init(cdNormal, 0)), nil);
    cmOpen : FileOpen('*.*');
    cmAbout: DoAboutBox;
   else
     exit;
  end;
  ClearEvent(Event);
end;

procedure TEditorApp.OutOfMemory;
begin
  MessageBox('Not enough memory for this operation.',
    nil, mfError + mfOkButton);
end;

end.
